using System;
using System.Configuration;
using System.Diagnostics;

namespace GoodsReceiptPO {
	public class Program {
		// app.config values
		private static string ServerType = ConfigurationManager.AppSettings["ServerType"];
		private static string ServerName = ConfigurationManager.AppSettings["ServerName"];
		private static string DatabaseName = ConfigurationManager.AppSettings["DatabaseName"];
		private static string DatabaseUserName = ConfigurationManager.AppSettings["DatabaseUserName"];
		private static string DatabasePassword = ConfigurationManager.AppSettings["DatabasePassword"];
		private static string SapUserName = ConfigurationManager.AppSettings["SapUserName"];
		private static string SapPassword = ConfigurationManager.AppSettings["SapPassword"];
		private static string PurchaseOrderEntry = ConfigurationManager.AppSettings["PurchaseOrderEntry"];
		private static string BinLocationAbsEntry = ConfigurationManager.AppSettings["BinLocationAbsEntry"];

		private static void Main(string[] args) {
			try {
				Console.WriteLine(" Connecting to company...");
				SAPbobsCOM.Company company = ConnectToCompany();
				Console.WriteLine();
				Console.WriteLine(string.Format("   Creating Goods Receipt PO... "));
				CreateGoodsReceiptPO(company);
				Console.WriteLine();
			} catch (Exception ex) {
				Console.WriteLine("   Error: " + ex.Message);
			}
			Console.ReadKey();
			Console.WriteLine();
			Console.Write(" Exiting... ");
		}

		private static void CreateGoodsReceiptPO(SAPbobsCOM.Company company) {
			int orderEntry;
			if (!int.TryParse(PurchaseOrderEntry, out orderEntry)) {
				throw new Exception("Invalid PurchaseOrderEntry '" + PurchaseOrderEntry + "' in app.config");
			}
			SAPbobsCOM.Documents purchaseOrder = (SAPbobsCOM.Documents)company.GetBusinessObject(SAPbobsCOM.BoObjectTypes.oPurchaseOrders);
			purchaseOrder.GetByKey(orderEntry);
			SAPbobsCOM.Documents grpo = (SAPbobsCOM.Documents)company.GetBusinessObject(SAPbobsCOM.BoObjectTypes.oPurchaseDeliveryNotes);
			grpo.CardCode = purchaseOrder.CardCode;
			grpo.CardName = purchaseOrder.CardName;
			int lineCounter = 0;
			for (int i = 0; i < purchaseOrder.Lines.Count; i++) {
				purchaseOrder.Lines.SetCurrentLine(i);
				if (purchaseOrder.Lines.Quantity == 0d) continue;
				SAPbobsCOM.Items item = (SAPbobsCOM.Items)company.GetBusinessObject(SAPbobsCOM.BoObjectTypes.oItems);
				item.GetByKey(purchaseOrder.Lines.ItemCode);
				double units = purchaseOrder.Lines.UnitsOfMeasurment;
				if (item.ManageSerialNumbers == SAPbobsCOM.BoYesNoEnum.tYES && units != 1d) continue;
				if (purchaseOrder.Lines.LineStatus != SAPbobsCOM.BoStatus.bost_Open) {
					throw new Exception("Line status must be open for all lines 'line: " + purchaseOrder.Lines.LineNum + "'");
				}
				if (grpo.Lines.Quantity != 0d) grpo.Lines.Add();
				grpo.Lines.BaseType = 22;
				grpo.Lines.BaseEntry = purchaseOrder.Lines.DocEntry;
				grpo.Lines.BaseLine = purchaseOrder.Lines.LineNum;
				grpo.Lines.ItemCode = purchaseOrder.Lines.ItemCode;
				grpo.Lines.WarehouseCode = purchaseOrder.Lines.WarehouseCode;
				if (item.ManageBatchNumbers == SAPbobsCOM.BoYesNoEnum.tYES) {
					grpo.Lines.BatchNumbers.BatchNumber = "B1";
					grpo.Lines.BatchNumbers.Quantity = purchaseOrder.Lines.Quantity * units;
				} else if (item.ManageSerialNumbers == SAPbobsCOM.BoYesNoEnum.tYES) {
					for (int j = 0; j < purchaseOrder.Lines.Quantity; j++) {
						if (grpo.Lines.SerialNumbers.Quantity > 0d) grpo.Lines.SerialNumbers.Add();
						grpo.Lines.SerialNumbers.InternalSerialNumber = "O" + orderEntry + "L" + (i + 1) + "S" + (j + 1);
						grpo.Lines.SerialNumbers.Quantity = 1 * units;
					}
				}
				if (!string.IsNullOrEmpty(BinLocationAbsEntry)) {
					int binAbsEntry;
					if (int.TryParse(BinLocationAbsEntry, out binAbsEntry) && binAbsEntry > 0) {
						grpo.Lines.BinAllocations.SetCurrentLine(0);
						if (item.ManageSerialNumbers == SAPbobsCOM.BoYesNoEnum.tYES) {
							for (int k = 0; k < grpo.Lines.SerialNumbers.Count; k++) {
								if (grpo.Lines.BinAllocations.Quantity > 0) grpo.Lines.BinAllocations.Add();
								grpo.Lines.BinAllocations.BinAbsEntry = binAbsEntry;
								grpo.Lines.BinAllocations.SerialAndBatchNumbersBaseLine = k;
								grpo.Lines.BinAllocations.Quantity = 1 * units;
							}
						} else {
							grpo.Lines.BinAllocations.BinAbsEntry = binAbsEntry;
							if (item.ManageBatchNumbers == SAPbobsCOM.BoYesNoEnum.tYES) {
								grpo.Lines.BinAllocations.SerialAndBatchNumbersBaseLine = 0;
							}
							grpo.Lines.BinAllocations.Quantity = purchaseOrder.Lines.Quantity * units;
						}
					}
				}
				grpo.Lines.Quantity = purchaseOrder.Lines.Quantity;
				lineCounter++;
			}
			Stopwatch stopwatch = Stopwatch.StartNew();
			int result = grpo.Add();
			stopwatch.Stop();
			if (result != 0) {
				Console.WriteLine(string.Format("   Failed (" + company.GetLastErrorDescription() + ")"));
			} else {
				Console.WriteLine("   Successfully done in " + stopwatch.ElapsedMilliseconds + " ms (" + lineCounter + " lines)");
			}
		}

		private static SAPbobsCOM.Company ConnectToCompany() {
			SAPbobsCOM.Company company = new SAPbobsCOM.Company();
			company.Server = ServerName;
			company.CompanyDB = DatabaseName;
			company.UserName = SapUserName;
			company.Password = SapPassword;
			company.UseTrusted = false;
			switch (ServerType) {
				case "HANA": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_HANADB; break;
				case "SQL2008": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_MSSQL2008; break;
				case "SQL2012": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_MSSQL2012; break;
				case "SQL2014": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_MSSQL2014; break;
				case "SQL2016": company.DbServerType = SAPbobsCOM.BoDataServerTypes.dst_MSSQL2016; break;
				default: throw new Exception("Invalid ServerType '" + ServerType + "' in app.config");
			}
			company.DbUserName = DatabaseUserName;
			company.DbPassword = DatabasePassword;
			int res = company.Connect();
			if (res != 0) {
				int err;
				string error;
				company.GetLastError(out err, out error);
				throw new Exception(" Connection to company failed: " + err.ToString() + " " + error);
			} else {
				Console.WriteLine(" Connection to company successful.");
			}
			return company;
		}
	}
}